import sys
from mathterpreter.main import interpret


def main():
    if not sys.argv:
        return print("pass an argument to calculate")

    print(interpret("".join(sys.argv[1:]) or sys.stdin.read()))


if __name__ == '__main__':
    main()



from mathterpreter.lexer import Lexer
from mathterpreter._parser import Parser


def interpret(string: str, use_decimals: bool = True):
    return Parser(Lexer(string, use_decimals).tokenize()).parse().evaluate()



from mathterpreter.exceptions import *
from mathterpreter.tokens import TokenType, Token
from mathterpreter.lexer import Lexer
from mathterpreter.nodes import *
from mathterpreter._parser import Parser
from mathterpreter.main import interpret


class MathError(Exception):
    pass


class MathSyntaxError(MathError):
    def __init__(self, reason="", visualisation=""):
        self.reason = reason
        self.visualisation = visualisation

    def __str__(self):
        return f"Syntax error:\n" \
               f"{self.reason}\n" \
               f"{self.visualisation if self.visualisation else ''}"



import typing
from dataclasses import dataclass
import math
from decimal import Decimal


@dataclass
class Node:
    def evaluate(self) -> float:
        pass


@dataclass
class NumberNode(Node):
    value: typing.Union[Decimal, float]

    def __repr__(self):
        return str(self.value)

    def evaluate(self):
        return self.value


@dataclass
class PolarityNode(Node):
    node: Node


@dataclass
class PositiveNode(PolarityNode):
    def __repr__(self):
        return f"(+{self.node})"

    def evaluate(self) -> float:
        return +(self.node.evaluate())


@dataclass
class NegativeNode(PolarityNode):
    def __repr__(self):
        return f"(-{self.node})"

    def evaluate(self) -> float:
        return -(self.node.evaluate())


@dataclass
class OperationNode(Node):
    first_operand: Node
    second_operand: Node


@dataclass
class AdditionNode(OperationNode):
    def __repr__(self):
        return f"({self.first_operand}+{self.second_operand})"

    def evaluate(self):
        return self.first_operand.evaluate() + self.second_operand.evaluate()


@dataclass
class SubtractionNode(OperationNode):
    def __repr__(self):
        return f"({self.first_operand}-{self.second_operand})"

    def evaluate(self) -> float:
        return self.first_operand.evaluate() - self.second_operand.evaluate()


@dataclass
class MultiplicationNode(OperationNode):
    def __repr__(self):
        return f"({self.first_operand}*{self.second_operand})"

    def evaluate(self) -> float:
        return self.first_operand.evaluate() * self.second_operand.evaluate()


@dataclass
class DivisionNode(OperationNode):
    def __repr__(self):
        return f"({self.first_operand}/{self.second_operand})"

    def evaluate(self) -> float:
        try:
            return self.first_operand.evaluate() / self.second_operand.evaluate()
        except ZeroDivisionError:
            return math.inf


@dataclass
class RootNode(OperationNode):
    def __repr__(self):
        return f"({self.first_operand}√{self.second_operand})"

    def evaluate(self) -> float:
        return self.second_operand.evaluate() ** (1/self.first_operand.evaluate())


@dataclass
class ExponentiationNode(OperationNode):
    def __repr__(self):
        return f"({self.first_operand}^{self.second_operand})"

    def evaluate(self) -> float:
        return self.first_operand.evaluate() ** self.second_operand.evaluate()



import typing
from dataclasses import dataclass
from decimal import Decimal


class TokenType:
    NUMBER = 0
    DECIMAL_PLACE = 1
    ADDITION_OPERATOR = 2
    SUBTRACTION_OPERATOR = 3
    MULTIPLICATION_OPERATOR = 4
    DIVISION_OPERATOR = 5
    SQRT_OPERATOR = 6
    POWER_OPERATOR = 7
    OPENING_BRACKET = 8
    CLOSING_BRACKET = 9


FORMATS = {

    TokenType.NUMBER: "{0.value}",
    TokenType.DECIMAL_PLACE: ".",
    TokenType.ADDITION_OPERATOR: "+",
    TokenType.SUBTRACTION_OPERATOR: "-",
    TokenType.MULTIPLICATION_OPERATOR: "*",
    TokenType.DIVISION_OPERATOR: "/",
    TokenType.SQRT_OPERATOR: "√",
    TokenType.POWER_OPERATOR: "^",
    TokenType.OPENING_BRACKET: "(",
    TokenType.CLOSING_BRACKET: ")"
}


@dataclass
class Token:
    type: TokenType
    value: typing.Optional[typing.Union[Decimal, float]] = None

    def __str__(self):
        return FORMATS[self.type].format(self)



from mathterpreter.nodes import *
from mathterpreter.tokens import TokenType, Token
from mathterpreter.exceptions import MathSyntaxError
from typing import List


class Parser:
    def __init__(self, tokens: List[Token]):
        self._tokens = iter(tokens)
        self.token_list = tokens
        self._token = None
        self._index = -1
        self._iterate_token()

    def _iterate_token(self):
        try:
            self._token = next(self._tokens)
            self._index += 1
        except StopIteration:
            self._token = None

    def parse(self):
        if self._token is None:
            return None

        result = self._addition_subtraction_base()

        return result

    def _addition_subtraction_base(self):
        if self._token is None:
            output = ''.join([z.__str__() for z in self.token_list])
            raise MathSyntaxError("Expression expected", f"{output}\n{'^^^'.rjust(len(output) + 3)}")
        result = self._multiplication_division()
        while self._token is not None:
            if self._token.type == TokenType.ADDITION_OPERATOR:
                self._iterate_token()
                result = AdditionNode(result, self._multiplication_division())
            elif self._token.type == TokenType.SUBTRACTION_OPERATOR:
                self._iterate_token()
                result = SubtractionNode(result, self._multiplication_division())
            else:
                break

        return result

    def _multiplication_division(self):
        result = self._exponentiation_root()

        while self._token is not None:
            if self._token.type == TokenType.MULTIPLICATION_OPERATOR:
                self._iterate_token()
                result = MultiplicationNode(result, self._exponentiation_root())
            elif self._token.type == TokenType.DIVISION_OPERATOR:
                self._iterate_token()
                result = DivisionNode(result, self._exponentiation_root())
            elif self._token.type == TokenType.OPENING_BRACKET:
                result = MultiplicationNode(result, self._literal_polarity())
            else:
                break

        return result

    def _exponentiation_root(self):
        result = self._literal_polarity()
        while self._token is not None:
            if self._token.type == TokenType.POWER_OPERATOR:
                self._iterate_token()
                result = ExponentiationNode(result, self._literal_polarity())
            elif self._token.type == TokenType.SQRT_OPERATOR:
                self._iterate_token()
                result = RootNode(result, self._literal_polarity())
            else:
                break
        return result

    def _literal_polarity(self):
        token = self._token
        if token.type == TokenType.OPENING_BRACKET:
            self._iterate_token()
            result = self._addition_subtraction_base()
            self._iterate_token()
            return result

        else:
            self._iterate_token()
            if token.type == TokenType.NUMBER:
                return NumberNode(token.value)

            elif token.type == TokenType.ADDITION_OPERATOR:
                return PositiveNode(self._literal_polarity())

            elif token.type == TokenType.SUBTRACTION_OPERATOR:
                return NegativeNode(self._literal_polarity())
            else:
                output = [z.__str__() for z in self.token_list]
                raise MathSyntaxError("Unexpected token",
                                      f"{''.join(output)}\n"
                                      f"{'^'.rjust(sum([len(z) for z in output[:self._index]]))}")



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    LONG_DESCRIPTION = "\n" + fh.read()

DESCRIPTION = "A lightweight and basic maths interpreter"
VERSION = "1.1.2"

setup(
    name="mathterpreter",
    version=VERSION,
    author="pjones123",
    url="https://github.com/pjones123/mathterpreter",
    license="MIT",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['dataclasses'],
    keywords=['python', 'math', 'maths'],
    classifiers=[
        "Programming Language :: Python :: 3.7"
    ],
    entry_points = {
        "console_scripts": ["mathterpreter=mathterpreter.__main__:main"]
    }
)



from decimal import Decimal
import math

from mathterpreter.tokens import Token, TokenType
from mathterpreter.exceptions import MathSyntaxError
from typing import List


def expand(compact_dict):
    result = {}
    for item in compact_dict:
        if isinstance(item, tuple):
            for key in item:
                result[key] = compact_dict[item]
        else:
            result[item] = compact_dict[item]
    return result


TOKENS = {
    "+": lambda: TokenType.ADDITION_OPERATOR,
    "-": lambda: TokenType.SUBTRACTION_OPERATOR,
    "*": lambda: TokenType.MULTIPLICATION_OPERATOR,
    "/": lambda: TokenType.DIVISION_OPERATOR,
    ("sqrt", "√"): lambda: TokenType.SQRT_OPERATOR,
    "^": lambda: TokenType.POWER_OPERATOR,
    "(": lambda: TokenType.OPENING_BRACKET,
    ")": lambda: TokenType.CLOSING_BRACKET,
}
CONSTANTS = {
    ("pi", "π"): lambda: math.pi,
    "e": lambda: math.e,
}

class Lexer:
    def __init__(self, string: str = "", use_decimals: bool = True):
        self.string = string
        self.use_decimals = use_decimals
        self.number_type = Decimal if use_decimals else float
        self.tokens: List[Token] = []
        self._expanded_tokens = expand(TOKENS)
        self._expanded_constants = expand(CONSTANTS)
        self._string_iterator = iter(string)
        self._character = None
        self._index = 0
        self._iterate_string()

    def _iterate_string(self, append=False):
        try:
            next_char = next(self._string_iterator)
            self._character = self._character + next_char if append else next_char
            self._index += 1
        except StopIteration:
            self._character = None

    def generate_tokens(self):
        while self._character is not None:
            found = False
            if self._character in (" ", "\t", "\n"):
                self._iterate_string()
                continue
            if self._character in self._expanded_tokens:
                token_types = self._expanded_tokens[self._character]()
                token_types = token_types if isinstance(token_types, list) else [token_types]
                for token_type in token_types:
                    yield token_type if isinstance(token_type, Token) else Token(token_type)
                self._iterate_string()
                found = True
            elif self._character in self._expanded_constants:
                constant = self._expanded_constants[self._character]()
                yield Token(TokenType.OPENING_BRACKET)
                yield Token(TokenType.NUMBER, self.number_type(constant))
                yield Token(TokenType.CLOSING_BRACKET)
                found = True
                self._iterate_string()
            elif self._character == "." or self._character.isdigit():
                yield self._get_number()
                found = True
            else:
                found = self._get_multichar(self._expanded_constants) or self._get_multichar(self._expanded_tokens)
            if not found:
                raise MathSyntaxError("Unsupported token",
                                      f"{self.string}\n{'^'.rjust(self.string.index(self._character) + 1)}")

    def _get_number(self):
        has_decimal_point = False
        has_exponent = False
        string = self._character
        self._iterate_string()
        while self._character is not None \
                and (self._character.isdigit() or self._character in (".", "E", "+", "-",)):
            if self._character == ".":
                if has_decimal_point:
                    raise MathSyntaxError("Second decimal point in the number",
                                          f"{self.string}\n"
                                          f"{'^'.rjust(self.string.index(self._character) + 1)}")
                else:
                    has_decimal_point = True

            if self._character == "E":
                if has_exponent:
                    raise MathSyntaxError("Second exponent in the number",
                                          f"{self.string}\n"
                                          f"{'^'.rjust(self.string.index(self._character) + 1)}")
                else:
                    has_exponent = True

            if self._character == "+" or self._character == "-":
                if string[-1] != "E":  # exponent is not a previous symbol
                    break

            string += self._character
            self._iterate_string()

        if string.startswith("."):
            string = "0" + string
        if string.endswith(".") or string.endswith("+") or string.endswith("-") or string.endswith("E"):
            string += "0"

        return Token(TokenType.NUMBER, self.number_type(string))

    def _get_multichar(self, tokens):
        complete = False
        found_complete = ""
        string_builder = ""
        string_iterator = iter(self.string[self._index-1:])
        while not complete:
            complete = True
            for token in tokens:
                if token.startswith(string_builder):
                    if token == string_builder and len(string_builder) > len(found_complete):
                        found_complete = string_builder
                    try:
                        string_builder += next(string_iterator)
                    except StopIteration:
                        complete = True
                        break
                    complete = False

        found = string_builder if string_builder in tokens else found_complete
        if found:
            for _ in range(len(found)-1):
                self._iterate_string(True)
        return bool(found)

    def tokenize(self):
        for token in self.generate_tokens():
            self.tokens.append(token)
        return self.tokens

    def __str__(self):
        return "\n".join([z.__str__() for z in self.tokens])
